import { h } from "preact"
import { useContext, useState } from "preact/hooks"
import { API, Rema } from "../../context/rema"
import Window from "../shared/window"
import { Button, Label, Field } from "../shared/button"
import { Arrow, Spin, ErrorCi } from "../shared/icon"
import { showCapability } from "../../util/pact"

const Sign = ({ cmd, f, onCancel }) => {
  const rema = useContext(Rema)
  const [ status, setStatus ] = useState(null)

  const attemptSignature = async () => {
    setStatus("")
    try {
      let res = await API.sign(cmd)
      if (res == undefined)
        setStatus("canceled or missing signature")
      else
        f(res)
    } catch (e) {
      console.error(e)
      setStatus(e.message)
    }
  }

  console.log(cmd)

  // TODO chainweaver icon
  return <div class="rows x4" style="min-width: 560px; max-width: 62vw;">
    <h2>Transaction</h2>
    <div class="rows x2">
      <h3>code</h3>
      <code>{cmd.pactCode}</code>
    </div>
    <div class="rows x2">
      <h3>wallet</h3>
      { status ? <div><Label class="error bg hor-margin" align="lowercase"><ErrorCi />{status}</Label></div> : null }
      <div class="rows">
        <Field
          class="important invert primary"
          onClick={ status != "" ? attemptSignature : null }
          align="lowercase">
          {status == "" ? <Spin /> : null } sign with chainweaver
        </Field>
        <Button class="important outline" onClick={ onCancel } align="lowercase">
          cancel
        </Button>
      </div>
    </div>
  </div>
}

// TODO listen tx inclusion in block
// https://github.com/kadena-io/kadenaswap/blob/e3958cd2254aacc7c9558af5836ff2ec8612e7ae/frontend/src/contexts/PactContext.js#L1003

const TransactionBroadcast = ({ hash, onClose }) =>
  <div class="rows x4">
    <h2>Transaction Submitted</h2>
    <div class="rows x2">
      <p>View in explorer:</p>
      <Button class="outline">{hash} <Arrow angle="45" /></Button>
    </div>
    <Button class="important primary invert" onClick={onClose} align="lowercase">
      close
    </Button>
  </div>

export default ({ cmd, onCancel, onClose, ...rest }) => {
  const rema = useContext(Rema)
  const [ txHash, setTxHash ] = useState(null)

  return <Window {...rest}>
    { !txHash
      ? <Sign cmd={cmd} onCancel={onCancel} f={async (signedCmd) => {
        await API.send(signedCmd)
        setTxHash(signedCmd.hash)
      }}/>
      : <TransactionBroadcast hash={txHash} onClose={onClose} />
    }
  </Window>
}
