import { h, Fragment } from "preact"
import { Arrow } from "../shared/icon"
import Arrays from "../../util/arrays"

const mapAlign = (children, align) =>
  align == "lowercase"
    ? Arrays.singleton(children)
      .map(c => typeof c == "string" ? <span class="lowercase">{c}</span> : c )
    : children

export const Button = ({ children, class: className = "", align = "", type = "button", ...rest }) => {
  const attrs = { ...rest, class: "button " + className }
  children = mapAlign(children, align)
  if (rest.href)
    return rest.href.startsWith("http")
      ? <a {...attrs} target="_blank">{ children } <Arrow angle="45" /></a>
      : <a {...attrs}>{ children }</a>
  else
    return <button type={type} {...attrs}>{ children }</button>
}

export const Label = ({ children, class: className = "", align = "", ...rest }) =>
  <span class={"label " + className} {...rest}>
    { mapAlign(children, align) }
  </span>

export const Field = ({ children, asButton = "", asLabel = "", class: className = "", ...rest}) =>
  (rest.href && rest.href.length) || rest.onClick
    ? <Button {...rest} class={className + " " + asButton}>{children}</Button>
    : <Label {...rest} class={className + " " + asLabel}>{children}</Label>
