import { positiveModulo, hasArithmeticOps } from "./misc"

export default class Arrays {

  static range = (start, end, step = Math.sign(end - start)) =>
    Array.from(
      { length: step != 0 ? Math.floor((end - start) / step) + 1 : 1 },
      (_, i) => start + i * step)

  static repeat = (x, n) =>
    Array.from({ length: n }, () => x)

  static singleton = (a) => Array.isArray(a) ? a : [ a ]
}

if (!("zip" in Array.prototype)) {

  Array.prototype.minimum = function() {
    return this.reduce((a, b) => Math.min(a, b))
  }

  Array.prototype.maximum = function() {
    return this.reduce((a, b) => Math.max(a, b))
  }

  Array.prototype.zip = function (f, ...s) {
    s = [ this, ...s.map(a => Array.isArray(a) ? a : Arrays.repeat(a, this.length)) ]
    return Array.from(
      { length: s.map(a => a.length).minimum() },
      (_, i) => f(...s.map(a => a[i])) )
  }

  Array.prototype.min = function (...s) {
    return this.zip(Math.min, ...s)
  }

  Array.prototype.max = function (...s) {
    return this.zip(Math.max, ...s)
  }

  Array.prototype.clamp = function (lo, hi) {
    return this.min(hi).max(lo)
  }

  Array.prototype.add = function (v) {
    return hasArithmeticOps(this[0])
      ? this.zip((a, b) => a + b, v)
      : this.zip((a, b) => a.add(b), v)
  }

  Array.prototype.subtract = function (v) {
    return hasArithmeticOps(this[0])
      ? this.zip((a, b) => a - b, v)
      : this.zip((a, b) => a.subtract(b), v)
  }

  Array.prototype.multiply = function (v) {
    return hasArithmeticOps(this[0])
      ? this.zip((a, b) => a * b, v)
      : this.zip((a, b) => a.multiply(b), v)
  }

  Array.prototype.divide = function (v) {
    return hasArithmeticOps(this[0])
      ? this.zip((a, b) => a / b, v)
      : this.zip((a, b) => a.divide(b), v)
  }

  Array.prototype.sum = function () {
    return this.reduce((x, y) => x + y, 0)
  }

  Array.prototype.dot = function (v) {
    return this.multiply(v).sum()
  }

  Array.prototype.lerp = function (v, k) {
    return this.multiply(1 - k).add(v.multiply(k))
  }

  Array.prototype.each = function (...args) {
    this.forEach(...args)
    return this
  }

  Array.prototype.scan = function (f, seed) {
    let xs = Array.from({ length: this.length })
    this.each((x, i) => seed = xs[i] = f(seed, x))
    return xs
  }

  Array.prototype.at = function (index) {
    return this[positiveModulo(index, this.length)]
  }

  Array.prototype.intercalate = function(x) {
    return Array.from({ length: this.length * 2 - 1 }, (_, i) =>
      i % 2 == 0 ? this[Math.trunc(i / 2)] : x)
  }

  Array.prototype.equals = function (xs) {
    if (!Array.isArray(xs) || xs.length != this.length) return false
    return this.every((y, i) => y == xs[i])
  }
}
