import BigDecimal from "./bigdecimal"

class Reference {
  constructor(o) {
    Object.assign(this, o)
  }

  static isRef = (o) => typeof o == "object" && "refSpec" in o
}

// TODO guards

export const toNative = (o) => {
  switch (typeof o) {
    case "object":
      const singleton = Object.keys(o).length == 1
      switch (true) {
        case Array.isArray(o):
          return o.map(toNative)
        case singleton && "decimal" in o:
          return new BigDecimal(o.decimal)
        case singleton && "int" in o:
          return BigInt(o.int)
        case singleton && "time" in o:
          return new Date(o.time)
        case Reference.isRef(o):
          return new Reference(o)
        default: return Object.fromEntries(
          Object.entries(o).map(([k, v]) => [k, toNative(v)])
        )
      }
    case "number":
      return new BigDecimal(o)
    case "boolean":
    case "string":
      return o
    default:
      console.error(o)
      throw `toNative: unexpected pact value "${o}"`
  }
}

const formatPactTime = date => date.toISOString().slice(0, -5) + "Z"

export const fromNative = o => {
  switch (typeof o) {
    case "boolean":
    case "string":
      return o
    case "bigint":
      return { int: o.toString() }
    case "object":
      switch (true) {
        case o instanceof Array:
          return o.map(fromNative)
        case o instanceof BigDecimal:
          return { decimal: o.toString() }
        case o instanceof Date:
          return { time: formatPactTime(o) }
        case o instanceof Reference:
          return o
        case o == null || o == undefined:
          throw `fromNative: unsupported value "null"`
        default: return Object.fromEntries(
          Object.entries(o).map(([k, v]) => [k, fromNative(v)])
        )
      }
    default:
      throw `fromNative: unexpected value "${o}"`
  }
}

export const toPact = o => {
  switch (typeof o) {
    case "boolean":
    case "string":
      return JSON.stringify(o)
    case "bigint":
      return o.toString()
    case "object":
      switch (true) {
        case o instanceof Array:
          return "[ " + o.map(toPact).join(" ") + " ]"
        case o instanceof BigDecimal:
          return o.format({ minimumDecimals: 1 })
        case o instanceof Date:
          return `(time "${formatPactTime(o)}")`
        case o instanceof Reference:
          throw "toPact: not implemented yet"
        case o == null || o == undefined:
          throw `toPact: unsupported value "null"`
        default: return "{ " +
          Object.entries(o).map(([k, v]) => `"${k}": ${toPact(v)}`).join(", ")
        + " }"
      }
    default:
      throw `toPact: unexpected value "${o}"`
  }
}

export const alignIndent = (code) =>
  code.replaceAll(code.match(/^\s*(\n\s*)/)[0], "\n").trim()

export const showCapability = (cap) =>
  "(" + cap.name + cap.args.map(x => " " + toPact(toNative(x))).join("") + ")"
